/* Copyright (C) 2001 to 2004 Chris Vine

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYING distributed with the source files.

*/

#ifndef PROG_DEFS_H
#define PROG_DEFS_H

#include "config.h"

// deal with any configuration issues arising from config.h

#ifndef HAVE_BOOL
typedef int bool;
const bool false = 0;
const bool true = 1;
#endif

#ifndef ENABLE_NLS
inline const char* gettext(const char* text) {
  return text;
}
#endif

// deal with libsigc++
#if GTKMM_VERSION == 20 
#include "sigc_compatibility.h"
#endif

// define some common exit codes

#define MEM_ERROR 10
#define CONFIG_ERROR 11
#define COM_ERROR 12
#define FILEOPEN_ERROR 13
#define PIPE_ERROR 14
#define TERM_ERROR 15
#define ARG_ERROR 16
#define EXEC_ERROR 17
#define FORK_ERROR 18

#define RC_FILE "efax-gtkrc"

#include <vector>
#include <string>
#include <glibmm/ustring.h>
#include <glibmm/thread.h>

using namespace std;    // until I get a Std C++ compliant compiler to test with

#define MAX_TEMP_NAME 12

/************************* threading issues *************************

1. This program uses certain functions which are not guaranteed by
   IEEE Std 1003.1 to be thread safe, and which are not protected by
   mutexes because this program only uses them in one thread.  These
   functions are:

     localtime() - used in the main (GUI) thread (efax_controller.cpp)
     readdir()   - used in the main (GUI) thread (fax_list.cpp)

     gethostbyaddr() ) used in the socket server thread (socket_server.cpp)
     gethostbyname() )

   If the program is modified to use any of these in a different thread,
   mutexes will need to be applied.

2. The last member of the Prog_config class below is a pointer to a mutex object
   allocated in int main() in main.cpp.  The mutex should be locked whenever:

   (a) the main (GUI) thread is modifying a member of the Prog_config class, or
   (b) any worker thread is reading the Prog_config object.

   No worker thread should modify a Prog_config class member, to save excessive
   locking in this program

*********************************************************************/

struct Prog_config {
  char* lock_file; // use a char* instead of std::string
                   // as we need to use it in a signal handler
  std::string fixed_font;
  std::string homedir;
  char rings;
  char receive_dirname[MAX_TEMP_NAME + 1];
  bool tone_dial;
  bool found_rcfile;
  bool GPL_flag;
  bool print_popup;
  bool sock_server;
  bool sock_popup;
  bool other_sock_client_address;
  bool fax_received_popup;
  bool fax_received_exec;
  Glib::ustring my_name;
  Glib::ustring my_number;
  Glib::ustring page_size;  // either 'a4' 'letter' or 'legal'
  Glib::ustring page_dim;
  Glib::ustring resolution;
  Glib::ustring print_cmd;
  Glib::ustring print_shrink;
  Glib::ustring ps_view_cmd;
  Glib::ustring logfile_name;
  Glib::ustring sock_server_port;
  Glib::ustring fax_received_prog;
  std::vector<std::string> parms;
  std::vector<std::string> permitted_clients_list;
  Glib::Mutex* mutex_p;
};

extern Prog_config prog_config;           // defined in main.cpp
Glib::ustring configure_prog(bool reread);// also defined in main.cpp (pass reread as true if
                                          // the program parameters are being reread)

// generic beep() function defined in main.cpp
void beep(void);

// general function for writing an error defined in mainwindow.cpp
// the message must not be larger than PIPE_BUF in size
ssize_t write_error(const char*);

// this function is defined in mainwindow.cpp, and will cause any
// message written to stderr to be taken by the in built error pipe
// in MainWindow.  Only call this in a child process after fork()ing.
int connect_to_stderr(void);

#endif
